import logging
import os

from bson.objectid import ObjectId
from pymongo import MongoClient


logger = logging.getLogger(__name__)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


class MongoDB:
    """
    Mongo DB Utility to help put data to database via
    different python applications
    """
    def __init__(self, force_db_name=False, collection_name=None):
        """
        :usage: Creates connection to db, creates DB and collection (DEFAULT/TESTING)
        """
        self._create_connection(force_db_name=force_db_name)
        if collection_name:
            self._create_collection(collection_name)

    def __repr__(self):
        return str(self.db)

    def _create_connection(self, test=False, force_db_name=False):
        """ create a database connection to the Mongo database specified by conf file
            :param test: to create a test db or default db
            :return: mongoclient object
            """
        try:
            host = os.environ.get("DATABASE_HOST")
            port = os.environ.get("DATABASE_PORT")
            db_name =  os.environ.get("DATABASE_NAME")
            username = os.environ.get('MONGO_INITDB_ROOT_USERNAME')
            password = os.environ.get('MONGO_INITDB_ROOT_PASSWORD')
            logger.info(">>> Connecting to {} DB <<<".format(db_name))
            self.client = MongoClient(
                host, int(port),
                username=username, password=password
            )
            if force_db_name:
                self._create_db(force_db_name)
            else:
                self._create_db(db_name)
            return self.client
        except Exception as e:
            print(e)

    def _create_db(self, db_name):
        """
        Creates a database
        :param db_name: Database name picked from config
        :return: mongo db instance
        """
        self._db_name = db_name
        self.db = self.client.get_database(db_name)
        return self.db

    def _create_collection(self, collection_name):
        """
        Creates a collection and adds the collection as an
        attribute to the class for easier access. 
        :param collection_name: name of the collection
        :return: mongodb collection instance
        """
        collection = self.db.get_collection(collection_name)
        setattr(self, collection_name, collection)
        return collection

    def fetch_by_id(self, _id, collection_name):
        """
        Fetches a record by the provided _id
        :param _id: PK for the record on which the search to be done
        :param collection_name: collection name  on which the search is to be done.
        :return: Identified record
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)
        document = collection.find_one({'_id': ObjectId(_id)})
        return document

    def fetch_many(self, collection_name, filter_dict):
        """
        Fetches multiple records based on the filter
        :param collection_name: name of the collection on which the filter is to be done
        :param filter_dict: filter condition dictionary, leave empty for all
        :return: list of identified records
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)
        document = collection.find(filter_dict)
        return list(document)

    def insert(self, key, value, collection_name):
        """
        Inserts a new record to mentioned collection
        :param key: key of the record
        :param value:  value of the record
        :param collection_name: name of the collection
        :return: id of the inserted object
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        obj = collection.insert_one({key: value})
        return obj.inserted_id

    def insert_record(self, data, collection_name):
        """
        Inserts a new record to mentioned collection
        :param data: dictionary of key value pairs
        :param collection_name: name of the collection
        :return: id of the inserted object
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        obj = collection.insert_one(data)
        return obj.inserted_id

    def update(self, _id, key, value, collection_name):
        """
        Updates an existing record
        :param _id: id of the intended record
        :param key: key to update
        :param value: value to update
        :param collection_name: name of the collection
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        myquery = {"_id": ObjectId(_id)}
        newvalues = {"$set": {key: value}}

        collection.update_one(myquery, newvalues)

    def create_or_update(self, key, value, collection_name, _id=None):
        """
        Insert new or update if existing record
        :param key: key for the operation
        :param value: value for the operation
        :param collection_name: name of the collection
        :param _id: _id to find the record if already existing
        :return: id of the object
        """
        doc = self.fetch_by_id(_id, collection_name)
        if doc:
            self.update(_id, key, value, collection_name)
            return _id
        else:
            obj_id = self.insert(key, value, collection_name)
            return obj_id

    def bulk_insert(self, value_list, collection_name):
        """
        Bulk insert for a collection
        :param value_list: list of all records
        :param collection_name: name of the collection
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        try:
            collection.insert_many(value_list)
        except Exception as e:
            print(e)
            raise Exception("Bulk Insert Failed")

    def delete(self, _id, collection_name):
        """
        Deletes a selected record
        :param _id: id of a selected record
        :param collection_name: name of the collection
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        try:
            collection.remove(ObjectId(_id))
        except:
            raise Exception("Unable to delete record: {}".format(_id))

    def truncate_collection(self, collection_name):
        """
        Truncates a collection by removing all records
        :param collection_name: name of the collection
        """
        try:
            collection = getattr(self, collection_name)
        except:
            collection = self._create_collection(collection_name)

        collection.remove()


import setuptools


try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements


def load_requirements(fname):
    reqs = parse_requirements(fname, session="test")
    requirements = [str(ir.requirement) for ir in reqs]
    return requirements


def long_description(fname):
    with open(fname) as f:
        return f.read()


setuptools.setup(
    name='unimi-common-tools',
    version='0.0.5',
    scripts=[],
    author="Binay Kumar Ray",
    author_email="binayray2009@gmail.com",
    description="UniMI Common Tools",
    long_description=long_description('README.md'),
    long_description_content_type="text/markdown",
    url="https://github.com/binayr/UniMI-common-tools",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=load_requirements('requirements.txt'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import os
import logging
from confluent_kafka import Producer, Consumer


logger = logging.getLogger(__name__)
CONFIG = {'bootstrap.servers': os.getenv('KAFKA_TOPIC_SERVER')}


class KafkaUtil:
    def __init__(self, conf=None):
        self.conf = conf or CONFIG
        self.producer = Producer(self.conf)

    def new_consumer(self):
        conf = {
            'group.id': 'mygroup',
            'auto.offset.reset': 'earliest',
            'auto.commit.interval.ms': 1000
        }
        conf.update(**self.conf)
        logger.info('Creating New Consumer')
        return Consumer(conf)

    def publish(self, topic_name, key, value):
        # Asynchronously produce a message, the delivery report callback
        # will be triggered from poll() above, or flush() below, when the message has
        # been successfully delivered or failed permanently.
        logger.info('Pulishing msg on topic {0} for {1}: {2}'.format(topic_name, key, value))
        self.producer.produce(topic_name, value.encode('utf-8'), key, callback=delivery_report)

        # Wait for any outstanding messages to be delivered and delivery report
        # callbacks to be triggered.
        self.producer.flush()

    def consume(self, consumer, topic, redirect, publish_to=None, publish_topic=None):
        consumer.subscribe([topic])
        logger.info('Consuming messages for :{}'.format(topic))

        while True:
            msg = consumer.poll(1.0)

            if msg is None:
                continue
            if msg.error():
                logger.info("Consumer error: {}".format(msg.error()))
                continue

            logger.info('Received message: {} for {}'.format(msg.value().decode('utf-8'), topic))
            msg = msg.value().decode('utf-8')
            logger.info("Extracted msg is: {}".format(str(msg)))
            logger.info("Calling {}".format(str(redirect.__name__)))
            try:
                op = redirect(msg)
            except Exception as e:
                logger.error(e)

            logger.info('post operation {}'.format(str(redirect.__name__)))
        
            if publish_to:
                self.publish(publish_to, publish_topic, op)
            
        consumer.close()


def delivery_report(err, msg):
    """ Called once for each message produced to indicate delivery result.
        Triggered by poll() or flush(). """
    if err is not None:
        logger.info('Message delivery failed: {}'.format(err))
    else:
        logger.info('Message delivered to {} [{}]'.format(msg.topic(), msg.partition()))



